@@ -14,11 +14,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.jasper;
 
 import java.io.File;
 import java.util.Enumeration;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
 
@@ -63,9 +63,9 @@ public final class EmbeddedServletOptions implements Options {
     private boolean keepGenerated = true;
 
     /**
-     * Should template text that consists entirely of whitespace be removed?
+     * How should template text that consists entirely of whitespace be handled?
      */
-    private boolean trimSpaces = false;
+    private TrimSpacesOption trimSpaces = TrimSpacesOption.FALSE;
 
     /**
      * Determines whether tag handler pooling is enabled.
@@ -115,7 +115,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Need to have this as is for versions 4 and 5 of IE. Can be set from
      * the initParams so if it changes in the future all that is needed is
-     * to have a jsp initParam of type ieClassId="<value>"
+     * to have a jsp initParam of type ieClassId="&lt;value>"
      */
     private String ieClassId = "clsid:8AD9C840-044E-11D1-B3E9-00805F499D93";
 
@@ -132,12 +132,12 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Compiler target VM.
      */
-    private String compilerTargetVM = "1.7";
+    private String compilerTargetVM = "1.8";
 
     /**
      * The compiler source VM.
      */
-    private String compilerSourceVM = "1.7";
+    private String compilerSourceVM = "1.8";
 
     /**
      * The compiler class name.
@@ -163,7 +163,7 @@ public final class EmbeddedServletOptions implements Options {
      * Java platform encoding to generate the JSP
      * page servlet.
      */
-    private String javaEncoding = "UTF8";
+    private String javaEncoding = "UTF-8";
 
     /**
      * Modification test interval.
@@ -199,19 +199,11 @@ public final class EmbeddedServletOptions implements Options {
      */
     private int jspIdleTimeout = -1;
 
-    /**
-     * System property that controls if the strict quoting rules are applied
-     * when parsing attribute values that use scriptlet expressions (<%=...%>).
-     */
-    private static final boolean STRICT_QUOTE_ESCAPING_DEFAULT= Boolean.parseBoolean(
-            System.getProperty(
-                    "org.apache.jasper.compiler.Parser.STRICT_QUOTE_ESCAPING",
-                    "true"));
     /**
      * Should JSP.1.6 be applied strictly to attributes defined using scriptlet
      * expressions?
      */
-    private boolean strictQuoteEscaping = STRICT_QUOTE_ESCAPING_DEFAULT;
+    private boolean strictQuoteEscaping = true;
 
     /**
      * When EL is used in JSP attribute values, should the rules for quoting of
@@ -238,19 +230,13 @@ public final class EmbeddedServletOptions implements Options {
         return quoteAttributeEL;
     }
 
-    /**
-     * Are we keeping generated code around?
-     */
     @Override
     public boolean getKeepGenerated() {
         return keepGenerated;
     }
 
-    /**
-     * Should template text that consists entirely of whitespace be removed?
-     */
     @Override
-    public boolean getTrimSpaces() {
+    public TrimSpacesOption getTrimSpaces() {
         return trimSpaces;
     }
 
@@ -259,138 +245,87 @@ public final class EmbeddedServletOptions implements Options {
         return isPoolingEnabled;
     }
 
-    /**
-     * Are we supporting HTML mapped servlets?
-     */
     @Override
     public boolean getMappedFile() {
         return mappedFile;
     }
 
-    /**
-     * Should class files be compiled with debug information?
-     */
     @Override
     public boolean getClassDebugInfo() {
         return classDebugInfo;
     }
 
-    /**
-     * Background JSP compile thread check interval
-     */
     @Override
     public int getCheckInterval() {
         return checkInterval;
     }
 
-    /**
-     * Modification test interval.
-     */
     @Override
     public int getModificationTestInterval() {
         return modificationTestInterval;
     }
 
-    /**
-     * Re-compile on failure.
-     */
     @Override
     public boolean getRecompileOnFail() {
         return recompileOnFail;
     }
 
-    /**
-     * Is Jasper being used in development mode?
-     */
     @Override
     public boolean getDevelopment() {
         return development;
     }
 
-    /**
-     * Is the generation of SMAP info for JSR45 debugging suppressed?
-     */
     @Override
     public boolean isSmapSuppressed() {
         return isSmapSuppressed;
     }
 
-    /**
-     * Should SMAP info for JSR45 debugging be dumped to a file?
-     */
     @Override
     public boolean isSmapDumped() {
         return isSmapDumped;
     }
 
-    /**
-     * Are Text strings to be generated as char arrays?
-     */
     @Override
     public boolean genStringAsCharArray() {
         return this.genStringAsCharArray;
     }
 
-    /**
-     * Class ID for use in the plugin tag when the browser is IE.
-     */
+    @Deprecated
     @Override
     public String getIeClassId() {
         return ieClassId;
     }
 
-    /**
-     * What is my scratch dir?
-     */
     @Override
     public File getScratchDir() {
         return scratchDir;
     }
 
-    /**
-     * What classpath should I use while compiling the servlets
-     * generated from JSP files?
-     */
     @Override
     public String getClassPath() {
         return classpath;
     }
 
-    /**
-     * Is generation of X-Powered-By response header enabled/disabled?
-     */
     @Override
     public boolean isXpoweredBy() {
         return xpoweredBy;
     }
 
-    /**
-     * Compiler to use.
-     */
     @Override
     public String getCompiler() {
         return compiler;
     }
 
-    /**
-     * @see Options#getCompilerTargetVM
-     */
     @Override
     public String getCompilerTargetVM() {
         return compilerTargetVM;
     }
 
-    /**
-     * @see Options#getCompilerSourceVM
-     */
     @Override
     public String getCompilerSourceVM() {
         return compilerSourceVM;
     }
 
-    /**
-     * Java compiler class to use.
-     */
     @Override
     public String getCompilerClassName() {
         return compilerClassName;
@@ -444,28 +379,16 @@ public final class EmbeddedServletOptions implements Options {
         return null;
     }
 
-    /**
-     * Should we include a source fragment in exception messages, which could be displayed
-     * to the developer ?
-     */
     @Override
     public boolean getDisplaySourceFragment() {
         return displaySourceFragment;
     }
 
-    /**
-     * Should jsps be unloaded if to many are loaded?
-     * If set to a value greater than 0 eviction of jsps is started. Default: -1
-     */
     @Override
     public int getMaxLoadedJsps() {
         return maxLoadedJsps;
     }
 
-    /**
-     * Should any jsps be unloaded when being idle for this time in seconds?
-     * If set to a value greater than 0 eviction of jsps is started. Default: -1
-     */
     @Override
     public int getJspIdleTimeout() {
         return jspIdleTimeout;
@@ -479,9 +402,10 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Create an EmbeddedServletOptions object using data available from
      * ServletConfig and ServletContext.
+     * @param config The Servlet config
+     * @param context The Servlet context
      */
-    public EmbeddedServletOptions(ServletConfig config,
-            ServletContext context) {
+    public EmbeddedServletOptions(ServletConfig config, ServletContext context) {
 
         Enumeration<String> enumeration=config.getInitParameterNames();
         while( enumeration.hasMoreElements() ) {
@@ -506,20 +430,17 @@ public final class EmbeddedServletOptions implements Options {
 
         String trimsp = config.getInitParameter("trimSpaces");
         if (trimsp != null) {
-            if (trimsp.equalsIgnoreCase("true")) {
-                trimSpaces = true;
-            } else if (trimsp.equalsIgnoreCase("false")) {
-                trimSpaces = false;
-            } else {
+            try {
+                trimSpaces = TrimSpacesOption.valueOf(trimsp.toUpperCase(Locale.ENGLISH));
+            } catch (IllegalArgumentException iae) {
                 if (log.isWarnEnabled()) {
-                    log.warn(Localizer.getMessage("jsp.warning.trimspaces"));
+                    log.warn(Localizer.getMessage("jsp.warning.trimspaces"), iae);
                 }
             }
         }
 
         this.isPoolingEnabled = true;
-        String poolingEnabledParam
-        = config.getInitParameter("enablePooling");
+        String poolingEnabledParam = config.getInitParameter("enablePooling");
         if (poolingEnabledParam != null
                 && !poolingEnabledParam.equalsIgnoreCase("true")) {
             if (poolingEnabledParam.equalsIgnoreCase("false")) {
@@ -643,8 +564,7 @@ public final class EmbeddedServletOptions implements Options {
             }
         }
 
-        String errBeanClass =
-            config.getInitParameter("errorOnUseBeanInvalidClassAttribute");
+        String errBeanClass = config.getInitParameter("errorOnUseBeanInvalidClassAttribute");
         if (errBeanClass != null) {
             if (errBeanClass.equalsIgnoreCase("true")) {
                 errorOnUseBeanInvalidClassAttribute = true;
@@ -658,12 +578,14 @@ public final class EmbeddedServletOptions implements Options {
         }
 
         String ieClassId = config.getInitParameter("ieClassId");
-        if (ieClassId != null)
+        if (ieClassId != null) {
             this.ieClassId = ieClassId;
+        }
 
         String classpath = config.getInitParameter("classpath");
-        if (classpath != null)
+        if (classpath != null) {
             this.classpath = classpath;
+        }
 
         /*
          * scratchdir
@@ -676,25 +598,18 @@ public final class EmbeddedServletOptions implements Options {
         if (dir != null) {
             scratchDir = new File(dir);
         } else {
-            // First try the Servlet 2.2 javax.servlet.context.tempdir property
             scratchDir = (File) context.getAttribute(ServletContext.TEMPDIR);
-            if (scratchDir == null) {
-                // Not running in a Servlet 2.2 container.
-                // Try to get the JDK 1.2 java.io.tmpdir property
-                dir = System.getProperty("java.io.tmpdir");
-                if (dir != null)
-                    scratchDir = new File(dir);
-            }
         }
-        if (this.scratchDir == null) {
+        if (scratchDir == null) {
             log.fatal(Localizer.getMessage("jsp.error.no.scratch.dir"));
             return;
         }
 
         if (!(scratchDir.exists() && scratchDir.canRead() &&
-                scratchDir.canWrite() && scratchDir.isDirectory()))
+                scratchDir.canWrite() && scratchDir.isDirectory())) {
             log.fatal(Localizer.getMessage("jsp.error.bad.scratch.dir",
                     scratchDir.getAbsolutePath()));
+        }
 
         this.compiler = config.getInitParameter("compiler");
 
@@ -787,8 +702,7 @@ public final class EmbeddedServletOptions implements Options {
                 this.strictQuoteEscaping = false;
             } else {
                 if (log.isWarnEnabled()) {
-                    log.warn(Localizer.getMessage("jsp.warning.strictQuoteEscaping",
-                            Boolean.toString(STRICT_QUOTE_ESCAPING_DEFAULT)));
+                    log.warn(Localizer.getMessage("jsp.warning.strictQuoteEscaping"));
                 }
             }
         }
